package eksempel.soap;

import eksempel.helper.ConstraintsHelper;
import eksempel.helper.FileHelper;
import eksempel.listener.BrowseAction;
import org.apache.commons.codec.binary.Base64;
import org.openoces.opensign.certificate.plugin.pkcs12.Pkcs12CertificateHandler;
import org.openoces.opensign.certificate.support.suncrypto.SunCryptoSupport;
import org.openoces.opensign.client.applet.CallBackHandler;
import org.openoces.opensign.client.applet.OS;
import org.openoces.opensign.client.applet.dialogs.DialogFactory;
import org.openoces.opensign.client.applet.dialogs.InputValidator;
import org.openoces.opensign.crypto.DefaultSignatureAlgorithm;
import org.openoces.opensign.xml.xmldsig.BasicSignatureGenerator;
import org.openoces.opensign.xml.xmldsig.SignatureGenerator;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URL;
import java.nio.charset.StandardCharsets;
import java.util.Properties;

/*
 * Denne klasse opretter et vindue til at tage imod input til indlæsning af XBRL dokumenter og
 * benytter derefter {@link RegisterXBRLInvoker} til at foretage kaldet til webservicen.
 * 
 * Denne klasse anvender BasicSignatureGenerator fra {@link openoces.org} til generering af XML Digital Signatur.
 * Denne generator skal anvendes da den genererede signatur valideres af en matchende klasse i webservicen. 
 */
public class RegisterXBRLFront {

    public static void main(String[] args) {
        SwingUtilities.invokeLater(() -> createAndShowGUI());
    }

    /**
     * Create the GUI and show it.  For thread safety,
     * this method should be invoked from the
     * event-dispatching thread.
     */
	static JFrame frame = null;
    private static void createAndShowGUI() {
    	
    	// Hoved ramme med titel
    	frame = new JFrame("XBRL eksempel klient");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        // Panel til input felter/boxe
        JPanel panel = new JPanel();
        panel.setLayout(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        c.fill = GridBagConstraints.HORIZONTAL;
        Insets insets = new Insets(3, 8, 3, 8);
        c.insets = insets;
        
        int gridy = 0;

        // Overskrift
        JLabel headLabel = new JLabel("Dialog til eksempel på implementering af Erhvervsstyrelsens system-til-system (WSDL) løsning");
        c.weightx = 1;
        c.gridwidth = 7;
        c.gridx = 0;
        c.gridy = gridy;
        panel.add(headLabel, c);

        JLabel separator0 = new JLabel(" ");
        c.weightx = 1;
        c.gridwidth = 7;
        c.gridx = 0;
        c.gridy = ++gridy;
        panel.add(separator0, c);

        // XBRL dokument vælger
        JLabel xbrlLabel = new JLabel("XBRL dokument");
        GridBagConstraints xbrlLabelConstraints = ConstraintsHelper.createConstraints();
        xbrlLabelConstraints.weightx = 1;
        xbrlLabelConstraints.gridwidth = 1;
        xbrlLabelConstraints.gridx = 0;
        xbrlLabelConstraints.gridy = ++gridy;
        panel.add(xbrlLabel, xbrlLabelConstraints);
        final JTextField xbrlFile = new JTextField(35);
        GridBagConstraints xbrlFileConstraints = ConstraintsHelper.createConstraints();
        xbrlFileConstraints.weightx = 1;
        xbrlFileConstraints.gridwidth = 5;
        xbrlFileConstraints.gridx = 1;
        xbrlFileConstraints.gridy = gridy;
        panel.add(xbrlFile, xbrlFileConstraints);
        JButton xbrlBrowse = new JButton("...");
        xbrlBrowse.addActionListener(new BrowseAction(frame, xbrlFile));
        GridBagConstraints xbrlBrowseConstraints = ConstraintsHelper.createConstraints();
        xbrlBrowseConstraints.weightx = 0.5;
        xbrlBrowseConstraints.gridwidth = 1;
        xbrlBrowseConstraints.gridx = 6;
        xbrlBrowseConstraints.gridy = gridy;
        panel.add(xbrlBrowse, xbrlBrowseConstraints);

        // Supplerende instans XBRL dokument vælger
        JLabel supplerendeInstansXbrlLabel = new JLabel("Supplerende instans XBRL");
        GridBagConstraints supplerendeInstansXbrlLabelConstraints= ConstraintsHelper.createConstraints();
        supplerendeInstansXbrlLabelConstraints.weightx = 1;
        supplerendeInstansXbrlLabelConstraints.gridwidth = 1;
        supplerendeInstansXbrlLabelConstraints.gridx = 0;
        supplerendeInstansXbrlLabelConstraints.gridy = ++gridy;
        panel.add(supplerendeInstansXbrlLabel, supplerendeInstansXbrlLabelConstraints);
        final JTextField supplerendeInstansXbrlFile = new JTextField(35);
        GridBagConstraints supplerendeInstansXbrlFileConstraints= ConstraintsHelper.createConstraints();
        supplerendeInstansXbrlFileConstraints.weightx = 1;
        supplerendeInstansXbrlFileConstraints.gridwidth = 5;
        supplerendeInstansXbrlFileConstraints.gridx = 1;
        supplerendeInstansXbrlFileConstraints.gridy = gridy;
        panel.add(supplerendeInstansXbrlFile, supplerendeInstansXbrlFileConstraints);
        JButton supplerendeInstansXbrlBrowse = new JButton("...");
        supplerendeInstansXbrlBrowse.addActionListener(new BrowseAction(frame, supplerendeInstansXbrlFile));
        GridBagConstraints supplerendeInstansXbrlBrowseConstraints= ConstraintsHelper.createConstraints();
        supplerendeInstansXbrlBrowseConstraints.weightx = 0.5;
        supplerendeInstansXbrlBrowseConstraints.gridwidth = 1;
        supplerendeInstansXbrlBrowseConstraints.gridx = 6;
        supplerendeInstansXbrlBrowseConstraints.gridy = gridy;
        panel.add(supplerendeInstansXbrlBrowse, supplerendeInstansXbrlBrowseConstraints);

        // Pdf vælger
        JLabel pdfLabel = new JLabel("PDF dokument (til ERST)");
        GridBagConstraints pdfLabelConstraints= ConstraintsHelper.createConstraints();
        pdfLabelConstraints.weightx = 1;
        pdfLabelConstraints.gridwidth = 1;
        pdfLabelConstraints.gridx = 0;
        pdfLabelConstraints.gridy = ++gridy;
        panel.add(pdfLabel, pdfLabelConstraints);
        final JTextField pdfFile = new JTextField(35);
        pdfFile.setText(System.getProperty("user.home") + "/testfiler/testpdf.pdf");

        GridBagConstraints pdfFileConstraints= ConstraintsHelper.createConstraints();
        pdfFileConstraints.weightx = 1;
        pdfFileConstraints.gridwidth = 5;
        pdfFileConstraints.gridx = 1;
        pdfFileConstraints.gridy = gridy;
        panel.add(pdfFile, pdfFileConstraints);
        JButton pdfBrowse = new JButton("...");
        pdfBrowse.addActionListener(new BrowseAction(frame, pdfFile));

        GridBagConstraints pdfBrowseConstraints= ConstraintsHelper.createConstraints();
        pdfBrowseConstraints.weightx = 0.5;
        pdfBrowseConstraints.gridwidth = 1;
        pdfBrowseConstraints.gridx = 6;
        pdfBrowseConstraints.gridy = gridy;
        panel.add(pdfBrowse, pdfBrowseConstraints);

        // XBRL Sender certifikat
        JLabel ocesSendLabel = new JLabel("Afsender PKCS#12 certifikat fil");
        GridBagConstraints ocesSendLabelConstraints= ConstraintsHelper.createConstraints();
        ocesSendLabelConstraints.weightx = 1;
        ocesSendLabelConstraints.gridwidth = 1;
        ocesSendLabelConstraints.gridx = 0;
        ocesSendLabelConstraints.gridy = ++gridy;
        panel.add(ocesSendLabel, ocesSendLabelConstraints);

        final JTextField ocesSendFile = new JTextField(35);
        GridBagConstraints ocesSendFileConstraints= ConstraintsHelper.createConstraints();
        ocesSendFile.setText(System.getProperty("user.home") + "/testfiler/MOCES_gyldig.p12");
        ocesSendFileConstraints.weightx = 1;
        ocesSendFileConstraints.gridwidth=5;
        ocesSendFileConstraints.gridx = 1;
        ocesSendFileConstraints.gridy = gridy;
        panel.add(ocesSendFile, ocesSendFileConstraints);

        JButton ocesSendBrowse = new JButton("...");
        GridBagConstraints ocesSendBrowseConstraints= ConstraintsHelper.createConstraints();
        ocesSendBrowse.addActionListener(new BrowseAction(frame, ocesSendFile));
        ocesSendBrowseConstraints.weightx = 0.5;
        ocesSendBrowseConstraints.gridwidth = 1;
        ocesSendBrowseConstraints.gridx = 6;
        ocesSendBrowseConstraints.gridy = gridy;
        panel.add(ocesSendBrowse, ocesSendBrowseConstraints);

        // XBRL Sender certifikat password
        JLabel ocesSendPwLabel = new JLabel("Afsender certifikat password");
        GridBagConstraints ocesSendPwLabelConstraints= ConstraintsHelper.createConstraints();
        ocesSendPwLabelConstraints.weightx = 1;
        ocesSendPwLabelConstraints.gridwidth = 1;
        ocesSendPwLabelConstraints.gridx = 0;
        ocesSendPwLabelConstraints.gridy = ++gridy;
        panel.add(ocesSendPwLabel, ocesSendPwLabelConstraints);

        final JPasswordField ocesSendPw = new JPasswordField(15);
        GridBagConstraints ocesSendPwConstraints= ConstraintsHelper.createConstraints();
        ocesSendPw.setText("Test1234");
        ocesSendPwConstraints.weightx = 1;
        ocesSendPwConstraints.gridwidth=2;
        ocesSendPwConstraints.gridx = 1;
        ocesSendPwConstraints.gridy = gridy;
        panel.add(ocesSendPw, ocesSendPwConstraints);

        // Token
        JLabel tokenLabel = new JLabel("Token");
        c.weightx = 1;
        c.gridwidth = 1;
        c.gridx = 0;
        c.gridy = ++gridy;
        panel.add(tokenLabel, c);
        final JTextField token = new JTextField(35);
        c.weightx = 1;
        c.gridwidth=2;
        c.gridx = 1;
        c.gridy = gridy;
        panel.add(token, c);

        // Destination vælger
        JLabel separator1 = new JLabel(" ");
        c.weightx = 1;
        c.gridwidth = 7;
        c.gridx = 0;
        c.gridy = ++gridy;
        panel.add(separator1, c);

        JLabel destLabel = new JLabel("Data destination");
        c.weightx = 1;
        c.gridwidth = 1;
        c.gridx = 0;
        c.gridy = ++gridy;
        panel.add(destLabel, c);
        final JRadioButton eogsBox = new JRadioButton("ERST", true);
        c.weightx = 1;
        c.gridwidth = 1;
        c.gridx = 1;
        c.gridy = gridy;
        panel.add(eogsBox, c);
        final JRadioButton dstBox = new JRadioButton("DST", false);
        c.weightx = 1;
        c.gridwidth = 1;
        c.gridx = 1;
        c.gridy = ++gridy;
        panel.add(dstBox, c);
        final JRadioButton skatBox = new JRadioButton("SKAT", false);
        c.weightx = 1;
        c.gridwidth = 1;
        c.gridx = 1;
        c.gridy = ++gridy;
        panel.add(skatBox, c);

		ButtonGroup destgroup = new ButtonGroup();
		destgroup.add(eogsBox);
		destgroup.add(dstBox);
		destgroup.add(skatBox);

        // service vælger
        JLabel separator2 = new JLabel(" ");
        c.weightx = 1;
        c.gridwidth = 7;
        c.gridx = 0;
        c.gridy = ++gridy;
        panel.add(separator2, c);

        JLabel serviceLabel = new JLabel("Service der rammes");
        c.weightx = 1;
        c.gridwidth = 1;
        c.gridx = 0;
        c.gridy = ++gridy;

        panel.add(serviceLabel, c);

        final JRadioButton localRadio = new JRadioButton("localhost (til udviklingsbrug)", false);
        c.weightx = 1;
        c.gridwidth = 1;
        c.gridx = 1;
        c.gridy = gridy++;
        panel.add(localRadio, c);
        final JRadioButton devRadio = new JRadioButton("archdev (ERST Dev)", true);
        c.weightx = 1;
        c.gridwidth = 1;
        c.gridx = 1;
        c.gridy = gridy++;
        panel.add(devRadio, c);
        final JRadioButton testRadio = new JRadioButton("archtest (ERST Test)", false);
        c.weightx = 1;
        c.gridwidth = 1;
        c.gridx = 1;
        c.gridy = gridy++;
        panel.add(testRadio, c);
        final JRadioButton preprodRadio = new JRadioButton("archpreprod (ERST Preprod)", true);
        c.weightx = 1;
        c.gridwidth = 1;
        c.gridx = 1;
        c.gridy = gridy++;
        panel.add(preprodRadio, c);
        final JRadioButton prodRadio = new JRadioButton("archprod (ERST Produktion!)", true);
        c.weightx = 1;
        c.gridwidth = 1;
        c.gridx = 1;
        c.gridy = gridy++;
        panel.add(prodRadio, c);

        ButtonGroup group = new ButtonGroup();
        group.add(localRadio);
        group.add(devRadio);

        group.add(testRadio);
        group.add(preprodRadio);
        group.add(prodRadio);

        // Angiv om transport via https
        JLabel separator3 = new JLabel(" ");
        c.weightx = 1;
        c.gridwidth = 7;
        c.gridx = 0;
        c.gridy = gridy;
        panel.add(separator3, c);

        JLabel transportLabel = new JLabel("Transport");
        c.weightx = 1;
        c.gridwidth = 1;
        c.gridx = 0;
        c.gridy = ++gridy;
        panel.add(transportLabel, c);
        final JCheckBox httpsBox = new JCheckBox("HTTPS", false);
        c.weightx = 1;
        c.gridwidth = 1;
        c.gridx = 1;
        c.gridy = gridy;
        panel.add(httpsBox, c);

        // Indlæs knap
        JButton indlaesXbrl = new JButton("Indlæs");
        GridBagConstraints indlaesXbrlConstraints= ConstraintsHelper.createConstraints();
        indlaesXbrlConstraints.weightx = 1;
        indlaesXbrlConstraints.gridwidth = 1;
        indlaesXbrlConstraints.gridx = 1;
        indlaesXbrlConstraints.gridy = ++gridy;
        indlaesXbrlConstraints.fill = GridBagConstraints.NONE;
        panel.add(indlaesXbrl, indlaesXbrlConstraints);

        // Ryd knap
        JButton ryd = new JButton("Ryd indtastning");
        c.weightx = 1;
        c.gridwidth = 1;
        c.gridx = 1;
        c.gridy = ++gridy;
        c.fill = GridBagConstraints.NONE;
        panel.add(ryd, c);

        // ActionListener for indlæs knap - check (minimalt) input og initier pakning, signering, og upload af data
        class IndlaesXBRLAction implements ActionListener {
            private final JFrame parent;

            public IndlaesXBRLAction(JFrame parent) {
                super();
                this.parent = parent;
            }

            public void actionPerformed(ActionEvent e) {
                // The XBRL, PDF for EogsS and OCES must have been loaded
                boolean allOk = xbrlFile.getText() != null && !xbrlFile.getText().equals("")
                        && (pdfFile.getText() != null && !pdfFile.getText().equals("") || !eogsBox.isSelected())
                        && ocesSendFile.getText() != null && !ocesSendFile.getText().equals("")
                        && ocesSendPw.getText() != null && !ocesSendPw.getText().equals("");

                if (!allOk) {
                    String errorTxt = "Der SKAL vælges\n\tXBRL dokument\n\tAfsender certifikat og password" +
                            "\n\tsamt PDF dokument hvis destination er ERST";
                    JOptionPane.showMessageDialog(parent, errorTxt, "Der mangler obligatoriske felter", JOptionPane.ERROR_MESSAGE);
                } else {
                    // Simpel fejl håndtering
                    String errorText = "";

                    // Hent XBRL indhold
                    String xbrl = "";
                    try {
                        xbrl = FileHelper.readTextFile(xbrlFile.getText());
                    } catch (IOException e1) {
                        errorText = "Kan ikke læse XBRL dokument: \n" + e1.getMessage();
                    }

                    // Hent supplerende instans XBRL indhold
                    String supplerendeInstansXbrl = "";
                    String supplerendeInstansFil = supplerendeInstansXbrlFile.getText().trim();
                    if (!supplerendeInstansFil.equals("")) {
                        try {
                            supplerendeInstansXbrl = FileHelper.readTextFile(supplerendeInstansFil);
                        } catch (IOException e2) {
                            errorText = "Kan ikke læse supplerende instans XBRL: \n" + e2.getMessage();
                        }
                    }

                    String pdf = null;
                    // Hent PDF indhold og Base64 det, hvis til ERST
                    if (eogsBox.isSelected()) {
                        try {
                            File file = new File(pdfFile.getText());
                            FileInputStream fos = new FileInputStream(file);
                            byte[] pdfBytes = new byte[(int) file.length()];
                            int n = fos.read(pdfBytes);
                            if (n != pdfBytes.length)
                                throw new IOException(n + " bytes read but expected " + pdfBytes.length);
                            pdf = Base64.encodeBase64String(pdfBytes);
                        } catch (IOException e1) {
                            errorText = "Kan ikke læse pdf dokument: \n" + e1.getMessage();
                        }
                    }

                    // Opret XML Digitale signature på baggrund af sender og godkender certifikater
                    String xmlDSigSender = null;
                    String supplerendeInstansXmlDSigSender = null;

                    try {
                        OS.setCryptoSupport((new SunCryptoSupport()));

                        // Instantier signatur generator
                        SignatureGenerator generator = new BasicSignatureGenerator();

                        // Sender signatur
                        // Hent sender certifikat password
                        String senderKeyPassword = ocesSendPw.getText();

                        // Opret ny CallBackHandler, der har en justeret promptPassword metode - den normale i oces pakken forventer input i Applet'en
                        CallBackHandler senderHandler = new CallBackHandler() {
                            private String password = null;

                            public void userMessage(String header, String message, boolean error) { }

                            public String promptPassword(String message, InputValidator validator) {
                                if (password == null) {
                                    password = message;
                                }
                                return password;
                            }

                            public URL getDocumentBase() { return null; }

                            public Frame getFrame() { return null;}

                            public void confirmedUserMessage(String header, String message) { }

                            public void clearUserMessage(boolean error) { }

                            public void alert(String message) { }

                            public String getParameter(String key) { return null; }

                            public DialogFactory getDialogFactory() { return null; }

                            public void repaint() { }
                        };

                        InputValidator inputValidator = new InputValidator() {
                            public String getErrorMsg() {
                                return "Test error msg";
                            }

                            public String getInfo() {
                                return "Test info";
                            }

                            public boolean isValid(String arg0) {
                                return true;
                            }
                        };

                        // Initialiser handler med password
                        senderHandler.promptPassword(senderKeyPassword, inputValidator);

                        // Hent sender Certificate
                        Pkcs12CertificateHandler senderCertHandler = new Pkcs12CertificateHandler(new File(ocesSendFile.getText()), senderKeyPassword.toCharArray());

                        // Sæt callback handler på certificatehandler
                        senderCertHandler.setCallBackHandler(senderHandler);

                        // Lav signatur - XBRL dokument
                        StringBuffer contentToBeSigned = new StringBuffer(xbrl);
                        StringBuffer senderSignature = generator.sign(senderCertHandler, senderKeyPassword.toCharArray(), contentToBeSigned, null, new Properties(), null, new DefaultSignatureAlgorithm());
                        // Fjern xml prolog fra signaturen
                        xmlDSigSender = senderSignature.substring(40);
                        System.out.println(senderSignature.substring(0, 40));

                        FileOutputStream fos = new FileOutputStream(xbrlFile.getText() + "sign.xml");
                        fos.write(xmlDSigSender.getBytes(StandardCharsets.UTF_8));
                        fos.close();

                        // Lav signatur - supplerende instans XBRL
                        if (supplerendeInstansXbrl != null && !supplerendeInstansXbrl.trim().equals("")) {
                            contentToBeSigned = new StringBuffer(supplerendeInstansXbrl);
                            senderSignature = generator.sign(senderCertHandler, senderKeyPassword.toCharArray(), contentToBeSigned, null, new Properties(), null, new DefaultSignatureAlgorithm());
                            // Fjern xml prolog fra signaturen
                            supplerendeInstansXmlDSigSender = senderSignature.substring(40);
                            System.out.println(senderSignature.substring(0, 40));

                            FileOutputStream fosSupplerendeInstans = new FileOutputStream(xbrlFile.getText() + "sign.xml");
                            fosSupplerendeInstans.write(supplerendeInstansXmlDSigSender.getBytes(StandardCharsets.UTF_8));
                            fosSupplerendeInstans.close();
                        }

                    } catch (Exception e1) {
                        errorText = "Kan ikke generere digitale signaturer: \n" + e1.getMessage();
                    }

                    // Saml input
                    RegisterXBRLInput input = new RegisterXBRLInput();
                    input.setSendDst(dstBox.isSelected() ? "Y" : "N");
                    input.setSendEogS(eogsBox.isSelected() ? "Y" : "N");
                    input.setSendSkat(skatBox.isSelected() ? "Y" : "N");
                    input.setPdf(pdf);
                    input.setSignedXbrl(xmlDSigSender);
                    input.setSignedSupplerendeInstans(supplerendeInstansXmlDSigSender);
                    input.setToken(token.getText());

                    // Test for fejl
                    if (!errorText.equals("")) {
                        JOptionPane.showMessageDialog(parent, errorText, "Fejl ved behandling af data:", JOptionPane.ERROR_MESSAGE);
                    } else {
                        // Kald webservice
                        RegisterXBRLInvoker invoker = new RegisterXBRLInvoker();
                        String errorMsg = null;
                        String result = null;
                        try {
                            frame.setCursor(new Cursor(Cursor.WAIT_CURSOR));
                            result = invoker.registerXBRL(input, httpsBox.isSelected(), prodRadio.isSelected(), preprodRadio.isSelected(),
                                    testRadio.isSelected(), devRadio.isSelected());
                            frame.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
                        } catch (Exception e1) {
                            errorMsg = e1.getMessage();
                            frame.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
                        }

                        // Vis resultatet i dialog
                        if (errorMsg != null) {
                            JTextArea text = new JTextArea(errorMsg, 0, 100);
                            text.setLineWrap(true);
                            text.setSize(new Dimension(480, 10));
                            text.setPreferredSize(new Dimension(480, text.getPreferredSize().height));
                            JOptionPane.showMessageDialog(parent, text, "Fejl ved kald af webservice", JOptionPane.ERROR_MESSAGE);
                        } else {
                            JTextArea text = new JTextArea(result, 0, 75);
                            text.setLineWrap(true);
                            text.setSize(new Dimension(480, 10));
                            text.setPreferredSize(new Dimension(480, text.getPreferredSize().height));
                            JOptionPane.showMessageDialog(parent, text, "Resultat af kald til webservice", JOptionPane.INFORMATION_MESSAGE);
                        }
                    }
                }

            }
        }

        indlaesXbrl.addActionListener(new IndlaesXBRLAction(frame));

        // ActionListener til ryd knap
        class RydAction implements ActionListener {
        	public RydAction(){
        		super();
        	}

			public void actionPerformed(ActionEvent e) {
				// Nulstil input
				xbrlFile.setText("");
                supplerendeInstansXbrlFile.setText("");
				pdfFile.setText("");
				ocesSendFile.setText("");
				ocesSendPw.setText("");
				eogsBox.setSelected(true);
				dstBox.setSelected(false);
				skatBox.setSelected(false);
				testRadio.setSelected(true);
            }
        }

        ryd.addActionListener(new RydAction());

        // Vis vinduet.
        frame.getContentPane().setLayout(new BorderLayout());
        frame.getContentPane().add(panel, BorderLayout.NORTH);
        frame.pack();
        frame.setLocationRelativeTo(null);
        frame.setVisible(true);
    }
}
