package eksempel.soap;

import eksempel.soap.webservice_xbrl.registerXbrlEkstern.RegisterXbrlEksternWS;
import eksempel.soap.webservice_xbrl.registerXbrlEkstern.RegisterXbrlEksternWSServiceLocator;
import eksempel.soap.webservice_xbrl.registerXbrlEkstern.WsReturnRegister;

import javax.xml.bind.DatatypeConverter;
import javax.xml.rpc.ServiceException;
import java.awt.*;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;
import java.io.File;
import java.io.FileOutputStream;

/**
 * Anvendes til at kalde webservices via de klasser der er genereret udfra den publiserede wsdl.
 */
public class RegisterXBRLInvoker {
    public String registerXBRL(RegisterXBRLInput registerXbrlInput, boolean useHttps, boolean useProd, boolean usePreprod, boolean useTest, boolean useDev) throws Exception {

        String endpointToUse;
        if (useProd) {
            endpointToUse = "://erst.virk.dk/ri/s2s/RegisterXbrlEksternWS";
        } else if (usePreprod) {
            endpointToUse = "://erstpreprod.virk.dk/ri/s2s/RegisterXbrlEksternWS";
        } else if (useTest) {
            endpointToUse = "://ersttest.virk.dk/ri/s2s/RegisterXbrlEksternWS";
        } else if (useDev) {
            endpointToUse = "://erstdev.virk.dk/ri/s2s/RegisterXbrlEksternWS";
        } else {
            endpointToUse = "://localhost.erstdev.dk/regnskab-indberetning-web/s2s/RegisterXbrlEksternWS";
        }

        System.out.println("Benytter endpoint: " + endpointToUse);


        // Resultat af webservice kaldet
        String result = "";

        try {
            // Hent stub til kald af webservicen
            RegisterXbrlEksternWSServiceLocator stubLocator = new RegisterXbrlEksternWSServiceLocator();
            if (useHttps) {
                // Sæt endpoint - i det publiserede wsdl er endpointet den interne esb adresse
                stubLocator.setEndpointAddress("RegisterXbrlEksternWSPort", "https" + endpointToUse);
            } else {
                // Sæt endpoint - i det publiserede wsdl er endpointet den interne adresse
                stubLocator.setEndpointAddress("RegisterXbrlEksternWSPort", "http" + endpointToUse);
            }
            RegisterXbrlEksternWS stub = stubLocator.getRegisterXbrlEksternWSPort();

            // Kald webservicen
            WsReturnRegister returnStatus;
            String invocationTime = "WS kaldet tog ";
            try {
                long t1 = System.currentTimeMillis();
                returnStatus = stub.registerXbrlEkstern(registerXbrlInput.getSignedXbrl(), registerXbrlInput.getSignedSupplerendeInstans(), registerXbrlInput.getPdf(),
                        registerXbrlInput.getSendEogS(), registerXbrlInput.getSendDst(), registerXbrlInput.getSendSkat(), registerXbrlInput.getToken());
                long duration = System.currentTimeMillis() - t1;
                invocationTime = invocationTime + (duration / 1000) + " sekunder\n";
            } catch (Exception e) {
                e.printStackTrace();
                throw new Exception(e.getMessage());
            }

            // Opsaml retur værdier
            String skatKvitMsg = "";
            String skatKvittering = returnStatus.getSkatKvittering();
            if (skatKvittering != null) {
                skatKvitMsg = "\nKvittering fra SKAT er modtaget og lagt i filen " + gemSkatKvittering(skatKvittering);
            }
            String skatKvit = returnStatus.getSkatKvittering() != null ? "\nKvittering fra SKAT er modtaget" : "";
            result = invocationTime +
                    "Code: " + returnStatus.getStatusCode() +
                    "\nText: " + returnStatus.getStatusText() +
                    "\nDetail:\n" + returnStatus.getStatusDetail() + skatKvitMsg +
                    "\nToken: " + returnStatus.getToken() +
                    "\nSagsnummer: " + returnStatus.getUniqueReportIdentifier();
            Clipboard clpbrd = Toolkit.getDefaultToolkit().getSystemClipboard();
            StringSelection stringSelection = new StringSelection(returnStatus.getUniqueReportIdentifier());
            clpbrd.setContents(stringSelection, null);
        } catch (ServiceException e) {
            e.printStackTrace();
        }

        return result;
    }

    /*
     * Gem base64 encoded PDF kvittering fra SKAT i temporær fil.
     */
    String gemSkatKvittering(String kvit) throws java.io.IOException {
        File file = File.createTempFile("skatKvittering_", ".pdf");
        byte[] bytes = DatatypeConverter.parseBase64Binary(kvit);
        FileOutputStream fos = new FileOutputStream(file);
        fos.write(bytes);
        fos.close();
        return file.getAbsolutePath();
    }
}
